//                                               -*- C++ -*-
/**
 *  @file  t_NormalFactory_std.cxx
 *  @brief The test file of class Normal for standard methods
 *
 *  (C) Copyright 2005-2010 EDF-EADS-Phimeca
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License.
 *
 *  This library is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  @author: $LastChangedBy: dutka $
 *  @date:   $LastChangedDate: 2010-02-04 16:44:49 +0100 (jeu. 04 févr. 2010) $
 *  Id:      $Id: t_NormalFactory_std.cxx 1473 2010-02-04 15:44:49Z dutka $
 */
#include <iostream>
#include "OT.hxx"
#include "OTtestcode.hxx"
#include "OStream.hxx"
#include "NormalFactory.hxx"
#include "Normal.hxx"
#include "Distribution.hxx"
#include "NumericalSample.hxx"
#include "NumericalPoint.hxx"

using namespace OT;
using namespace OT::Test;
using namespace OT::Base::Common;
using namespace OT::Base::Stat;
using namespace OT::Base::Type;
using namespace OT::Uncertainty::Distribution;
using namespace OT::Uncertainty::Model;


int main(int argc, char *argv[])
{
  TESTPREAMBLE;
  OStream fullprint(std::cout);
  setRandomGenerator();  

  try {
    UnsignedLong dim(3);
    NumericalPoint mean(dim);
    NumericalPoint sigma(dim);
    CorrelationMatrix R(dim);
    for (UnsignedLong i = 0; i < dim; i++)
      {
	mean[i] = i + 0.5;
	sigma[i] = 2 * i + 1.0;
	for (UnsignedLong j = 0; j < i; j++)
	  {
	    R(i, j) = 0.5 * (1.0 + i) / dim;
	  }
      }
    Normal distribution(mean, sigma, R);
    UnsignedLong size(10000);
    NumericalSample sample(distribution.getNumericalSample(size));
    NormalFactory factory;
    Distribution estimatedDistribution(factory.buildImplementation(sample));
    fullprint << "Distribution          =" << distribution << std::endl;
    fullprint << "Estimated distribution=" << estimatedDistribution << std::endl;
  }
  catch (TestFailed & ex) {
    std::cerr << ex << std::endl;
    return ExitCode::Error;
  }


  return ExitCode::Success;
}
