//                                               -*- C++ -*-
/**
 *  @file  t_QuadraticCumul_std.cxx
 *  @brief The test file of class QuadraticCumul for the standard methods
 *
 *  (C) Copyright 2005-2010 EDF-EADS-Phimeca
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License.
 *
 *  This library is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  @author: $LastChangedBy: dutka $
 *  @date:   $LastChangedDate: 2010-02-04 16:44:49 +0100 (jeu. 04 févr. 2010) $
 *  Id:      $Id: t_QuadraticCumul_std.cxx 1473 2010-02-04 15:44:49Z dutka $
 */
#include <iostream>
#include <sstream>
#include <exception>
#include "OT.hxx"
#include "OTtestcode.hxx"
#include "OStream.hxx"
#include "NumericalMathFunction.hxx"
#include "QuadraticCumul.hxx"
#include "CorrelationMatrix.hxx"
#include "Normal.hxx"
#include "NumericalPoint.hxx"
#include "RandomVector.hxx"
#include "Exception.hxx"

using namespace OT;
using namespace OT::Test;
using namespace OT::Base::Common;
using namespace OT::Base::Common;
using namespace OT::Base::Type;
using namespace OT::Base::Func;
using namespace OT::Base::Stat;
using namespace OT::Uncertainty::Algorithm;
using namespace OT::Uncertainty::Model;
using namespace OT::Uncertainty::Distribution;

int main(int argc, char *argv[])
{
  TESTPREAMBLE;
  OStream fullprint(std::cout);

  try {

    // we test here the formulas on the identity function in R^4

    /* We create a NumericalMathFunction */
    NumericalMathFunction myFuncIdentity("TestIdentity");
    /* We create a distribution */
    UnsignedLong dim(myFuncIdentity.getInputDimension());
    NumericalPoint meanPoint(dim, 0.1);
    NumericalPoint sigma(dim, 1.0);
    CorrelationMatrix R(dim);
    for (UnsignedLong i = 0; i < dim; i++)
      {
	R(i, i) = 1.0;
      }
    for (UnsignedLong i = 1; i < dim; i++)
      {
	R(i, i - 1) = 0.25;
      }
    Normal distribution(meanPoint, sigma, R);

    /* We create a distribution-based RandomVector */
    RandomVector X(distribution);
    fullprint << "InputDistribution = " << distribution << std::endl;

    // We create a  composite vector 
    RandomVector W(myFuncIdentity, X);

    // We create a quadraticCumul algorithm
    QuadraticCumul myQuadraticCumulIdentity(W);
    // We compute the several elements provided by the quadratic cumul algorithm
    fullprint << "First order mean=" << myQuadraticCumulIdentity.getMeanFirstOrder() << std::endl;
    fullprint << "Second order mean=" << myQuadraticCumulIdentity.getMeanSecondOrder() << std::endl;
    fullprint << "Covariance=" << myQuadraticCumulIdentity.getCovariance() << std::endl;
  }

  catch (OT::Base::Common::NotSymmetricDefinitePositiveException & ex) {
    std::cerr << ex << std::endl;
    return ExitCode::Error;
  }
  
  catch (TestFailed & ex) {
    std::cerr << ex << std::endl;
    return ExitCode::Error;
  }

  return ExitCode::Success;
}
